(* file: line.ml
 * author: Bob Muller
 * date: November 30, 2014
 * 
 * This file contains a specification (signature) and the
 * implementation (structure) of a simple ADT for lines in
 * the 2D plane. The specification is usually contained in
 * a separate .mli file. See the implementation in ../file/
*)
    type t = {p1 : Point.t; p2: Point.t};;

    let make p1 p2 = {p1=p1; p2=p2};;

    let getP1 line = line.p1;;
    let getP2 line = line.p2;;

    let toString line =
      let p1s = Point.toString (getP1 line) in
      let p2s = Point.toString (getP2 line) 
      in
	"{p1=" ^ p1s ^ "; p2=" ^ p2s ^ "}";;

    let length line =
      let (x1, y1) = (Point.getX(getP1 line), Point.getY(getP1 line)) in
      let (x2, y2) = (Point.getX(getP2 line), Point.getY(getP2 line)) in
      let rise = (y2 -. y1) in
      let run =  (x2 -. x1) 
      in
	sqrt(rise ** 2.0 +. run ** 2.0);;

    let midpoint line =
      let (x1, y1) = (Point.getX(getP1 line), Point.getY(getP1 line)) in
      let (x2, y2) = (Point.getX(getP2 line), Point.getY(getP2 line)) 
      in
	Point.make ((x1 +. x2) /. 2.0) ((y1 +. y2) /. 2.0);;

    let compareTo line1 line2 =
      let l1mp = midpoint line1 in
      let l2mp = midpoint line2 
      in
	Point.compareTo l1mp l2mp;;

    let equal line1 line2 = (compareTo line1 line2) == 0;;

    let move line dx dy =
      let p1 = Point.move (getP1 line) dx dy in
      let p2 = Point.move (getP2 line) dx dy 
      in
	make p1 p2;;

